@@ -34,15 +34,27 @@ get_eps_T = function (DLpar, country, tfr_matrix, start, lambda, p1, p2)
     return(eps_T)
 }
 
+#get.dl.index <- function(country, meta) {
+#	index <- meta$start_c[country] : meta$lambda_c[country]
+#	if(is.null(meta$suppl.data) || meta$has.suppl.data[country]) return(index)
+#	return(index + mcmc$meta$suppl.data$T_end)
+#}
+
+get.eps.T <- function (DLpar, country, meta) 
+{
+    tfr <- get.observed.tfr(country, meta)[meta$start_c[country]:meta$lambda_c[country]]
+    ldl <- length(tfr)-1
+    dl <- DLcurve(DLpar, tfr[1:ldl], meta$dl.p1, meta$dl.p2)
+    return(tfr[2:(ldl+1)] - tfr[1:ldl] + dl)
+}
+
 get_eps_T_all <- function (mcmc) {
-	eps_Tc <- matrix(NA, mcmc$meta$T_end-1, mcmc$meta$nr_countries)
+	suppl.T <- if(!is.null(mcmc$meta$suppl.data$regions)) mcmc$meta$suppl.data$T_end else 0
+	eps_Tc <- matrix(NA, mcmc$meta$T_end-1 + suppl.T, mcmc$meta$nr_countries)
     for (country in mcmc$meta$id_DL){
     	theta <- c((mcmc$U_c[country]-mcmc$Triangle_c4[country])*exp(mcmc$gamma_ci[country,])/                                     
                                 sum(exp(mcmc$gamma_ci[country,])), mcmc$Triangle_c4[country], mcmc$d_c[country])
-        eps_Tc[mcmc$meta$start_c[country]:(mcmc$meta$lambda_c[country]-1), country] <- 
-                                        get_eps_T(theta, country, mcmc$meta$tfr_matrix, 
-                                                          mcmc$meta$start_c[country], 
-                                                          mcmc$meta$lambda_c[country], mcmc$meta$dl.p1, mcmc$meta$dl.p2)
+        eps_Tc[mcmc$meta$start_c[country]:(mcmc$meta$lambda_c[country]-1), country] <- get.eps.T(theta, country, mcmc$meta)
         }
     return(eps_Tc)
 }
@@ -50,7 +62,7 @@ get_eps_T_all <- function (mcmc) {
 find.lambda.for.one.country <- function(tfr, T_end) {
 	lambda <- T_end
 	if ( sum(tfr<2, na.rm=TRUE)>2 ){
-		period = 3
+		period <- .get.T.start.end(tfr)[1]+2
 		while (period<=T_end){
 			if (( (tfr[period] - tfr[period-1]) >0 )& 
                     ( (tfr[period-1] - tfr[period-2]) >0) & 
@@ -66,65 +78,106 @@ find.lambda.for.one.country <- function(tfr, T_end) {
 	return(lambda)
 }
 
-get.lambda <- function(tfr_matrix, T_end_c){
-	# find lambda_c's based on definition
-	# tfr increased twice, below 2
-    nr_countries = length(tfr_matrix[1,])
-    lambda_c <- T_end_c
-    for (country in 1:nr_countries)
-    	lambda_c[country] <- find.lambda.for.one.country(tfr_matrix[, country], T_end_c[country])
-	return(lambda_c)
+
+
+.get.T.start.end <- function(tfr) {
+	# Return first index after NAs at the beginning of the time series
+	isna.tfr <- is.na(tfr)
+	start <- if(sum(isna.tfr) > 0 && isna.tfr[1]) which(diff(cumsum(isna.tfr))==0)[1]+1 else 1
+	return(c(start, sum(!isna.tfr) + start - 1))
+} 
+
+get.observed.tfr <- function(country.index, meta, matrix.name='tfr_matrix', matrix.name.suppl=matrix.name)
+	return(get.observed.with.supplemental(country.index, meta[[matrix.name]], meta$suppl.data, matrix.name.suppl))
+
+get.observed.with.supplemental <- function(country.index, matrix, suppl.data, matrix.name='tfr_matrix') {
+	data <- matrix[,country.index]
+	if(!is.null(suppl.data[[matrix.name]])) {
+    	supp.c.idx <- suppl.data$index.from.all.countries[country.index]
+    	if(is.na(supp.c.idx)) {sdata <- rep(NA, nrow(suppl.data[[matrix.name]])); names(sdata) <- rownames(suppl.data[[matrix.name]])}
+    	else sdata <- suppl.data[[matrix.name]][,supp.c.idx]
+    	data <- c(sdata, data)
+    }
+	return(data)
 }
 
-
-find.tau.and.DLcountries = function(tfr_matrix, min.TFRlevel.for.start.after.1950 = 5.5, 
-												max.diff.local.and.global.max.for.start.at.loc = 0.5) 
+find.tau.lambda.and.DLcountries <- function(tfr_matrix, min.TFRlevel.for.start.after.1950 = 5, #5.5, 
+												max.diff.local.and.global.max.for.start.at.loc = 0.53, #0.5,
+												delta.for.local.max = 0.001,
+												suppl.data=NULL) 
 # gets tau_c and puts NAs before tau_c
-# gets ids of DL (where decline has been observed), excl countries with min <1 (macao and HK)
+# gets ids of DL (where decline has been observed)
 # and divides those into early and not early
+# find lambda_c's based on definition tfr increased twice, below 2
 {
-    T_end <- length(tfr_matrix[, 1])
+    T_end <- dim(tfr_matrix)[1]
     nr_countries <- dim(tfr_matrix)[2]
-    T_end_c <- rep(T_end, nr_countries)
-    decr_matrix = -tfr_matrix[-1, ] + tfr_matrix[-T_end, ]
-    tau_c = rep(NA, nr_countries)
+    T_end_c <- lambda_c <-rep(T_end, nr_countries)
+    #has.suppl <- rep(FALSE, nr_countries)
+    T.suppl <- if(is.null(suppl.data$regions)) 0 else dim(suppl.data$tfr_matrix)[1]
+    tau_c <- start_c <- rep(NA, nr_countries)
     for (country in 1:nr_countries) {
-    	T_end_c[country] = sum(!is.na(tfr_matrix[,country]))
-    	local_max_indices = rep(NA, T_end_c[country])
-        does_tfr_decrease = ifelse(diff(tfr_matrix[1:T_end_c[country],country]) < 0, 1, 0)
-        local_max_indices[1] = does_tfr_decrease[1]
+    	data <- get.observed.with.supplemental(country, tfr_matrix, suppl.data)
+    	has.suppl <- length(data) > T_end && !is.na(suppl.data$index.from.all.countries[country])
+    	# ignoring NAs at the beginning
+    	T.start.end <- .get.T.start.end(data)
+    	T.start <- T.start.end[1]
+    	T_end_c[country] = T.start.end[2]
+    	lT <- T_end_c[country] - T.start + 1
+    	local_max_indices <- rep(NA, lT)
+    	d <- diff(data[T.start:T_end_c[country]])
+        does_tfr_decrease <- ifelse(d < delta.for.local.max, 1, 0)
+        local_max_indices[1] <- does_tfr_decrease[1]
    		# in middle only a local max if increase is followed by decrease
-        local_max_indices[-c(1, T_end_c[country])] = diff(does_tfr_decrease)
+        local_max_indices[-c(1, lT)] = diff(does_tfr_decrease)
    		# at end local max if preceded by an increase 
-        local_max_indices[T_end_c[country]] = 1 - does_tfr_decrease[T_end_c[country] - 1]
- 		value_global_max = max(tfr_matrix[, country], na.rm = TRUE)
- 		max_index = max(seq(1, T_end_c[country]) * (local_max_indices >
-            0) * (ifelse(tfr_matrix[1:T_end_c[country], country] >
-            value_global_max - max.diff.local.and.global.max.for.start.at.loc, 1, 0)))
-        tau_c[country] = max_index
-        if ((tfr_matrix[tau_c[country], country] < min.TFRlevel.for.start.after.1950)) {
-            tau_c[country] = -1
+        local_max_indices[lT] = 1 - does_tfr_decrease[lT - 1]
+ 		value_global_max = max(data, na.rm = TRUE)
+ 		max_index <- max(seq(T.start, T_end_c[country]) * (local_max_indices > 0) * 
+ 						ifelse(data[T.start:T_end_c[country]] >
+            				value_global_max - max.diff.local.and.global.max.for.start.at.loc, 1, 0))
+        # move the point to the right if there are more recent points with the same values
+        is.same <- c(0, ifelse(abs(d) < delta.for.local.max, 1, 0), 0)
+        cs.same <- cumsum(is.same[(max_index-T.start+1):(lT+1)])
+        max_index <- max_index + cs.same[min(which(diff(cs.same)==0))]
+        tau_c[country] <- max_index
+        start_c[country] <- tau_c[country]
+
+        if (as.integer(names(data)[T.start]) > 1855 && (data[tau_c[country]] < min.TFRlevel.for.start.after.1950)) {
+        	tau_c[country] <- -1
+        	start_c[country] <- which(!is.na(data))[1] # first data point that is not NA
         }
-        if (tau_c[country] > 1) {
-            tfr_matrix[1:(tau_c[country] - 1), country] = NA
+        if (tau_c[country] > 1 && T.suppl > 0 && has.suppl) 
+        	suppl.data$tfr_matrix[1:min(tau_c[country] - 1, T.suppl),suppl.data$index.from.all.countries[country]] <- NA
+        if(tau_c[country] > T.suppl + 1)
+            tfr_matrix[1:(tau_c[country] - 1 - T.suppl), country] <- NA
+
+        lambda_c[country] <- find.lambda.for.one.country(data, T_end_c[country])
+        if (lambda_c[country] < T_end_c[country]) { # set NA all values between lambda_c and T_c_end
+         	if(lambda_c[country] < T.suppl) {
+         		suppl.data$tfr_matrix[(lambda_c[country] + 1):min(T.suppl, T_end_c[country]),
+         										suppl.data$index.from.all.countries[country]] <- NA
+         		if(T_end_c[country] > T.suppl) tfr_matrix[1:(T_end_c[country]-T.suppl),country] <- NA
+         	} else tfr_matrix[(lambda_c[country] - T.suppl + 1):(T_end_c[country]-T.suppl),country] <- NA
         }
     }
 
-    id_Tistau = seq(1, nr_countries)[tau_c == T_end_c]
+    id_Tistau <- seq(1, nr_countries)[tau_c == T_end_c]
         # not needed in fit
-    id_DL = seq(1, nr_countries)[(tau_c != T_end_c)
+    id_DL <- seq(1, nr_countries)[(tau_c != T_end_c)
     #& (apply(tfr_matrix, 2, min, na.rm = TRUE) >1) # excludes Macao and Hong Kong
                 ]
         # for par in BHM
-   id_early = seq(1, nr_countries)[(tau_c == -1)
+   id_early <- seq(1, nr_countries)[(tau_c == -1)
     #& (apply(tfr_matrix, 2, min, na.rm = TRUE) >1)
     			]
     # needed for which U_c's to update, this is updated in mcmc
    id_notearly = seq(1, nr_countries)[(tau_c != -1)  & (tau_c != T_end_c)
     #& (apply(tfr_matrix, 2, min, na.rm = TRUE) >1)
     			]
-    return(list(tau_c = tau_c, id_Tistau = id_Tistau, id_DL = id_DL, id_early = id_early,
-        		id_notearly = id_notearly, tfr_matrix = tfr_matrix, T_end_c=T_end_c
+    return(list(tau_c = tau_c,  id_Tistau = id_Tistau, id_DL = id_DL, id_early = id_early,
+        		id_notearly = id_notearly, tfr_matrix = tfr_matrix, T_end_c=T_end_c, 
+        		lambda_c=lambda_c, start_c=start_c, suppl.matrix=suppl.data$tfr_matrix
          ))
 }
 
@@ -143,7 +196,8 @@ mcmc.meta.ini <- function(...,
 	mcmc.input$start.year <- start.year
 	mcmc.input$present.year <- present.year
 	mcmc.input$wpp.year <- wpp.year
-	tfr.with.regions <- set_wpp_regions(start.year=start.year, present.year=present.year, 						wpp.year=wpp.year, my.tfr.file = my.tfr.file, verbose=verbose)
+	tfr.with.regions <- set_wpp_regions(start.year=start.year, present.year=present.year, wpp.year=wpp.year, 
+										my.tfr.file = my.tfr.file, verbose=verbose)
 
 	meta <- do.meta.ini(mcmc.input, tfr.with.regions, my.tfr.file=my.tfr.file, 
 						proposal_cov_gammas=proposal_cov_gammas, verbose=verbose)
@@ -153,48 +207,43 @@ mcmc.meta.ini <- function(...,
 	
 do.meta.ini <- function(meta, tfr.with.regions, my.tfr.file=NULL, proposal_cov_gammas = NULL, 
 						use.average.gammas.cov=FALSE, burnin=200, verbose=FALSE) {
-	results_tau <- find.tau.and.DLcountries(tfr.with.regions$tfr_matrix)
+	results_tau <- find.tau.lambda.and.DLcountries(tfr.with.regions$tfr_matrix, suppl.data=tfr.with.regions$suppl.data)
 	tfr_matrix_all <- tfr.with.regions$tfr_matrix_all
 	tfr_matrix_observed <- tfr.with.regions$tfr_matrix
 	updated.tfr.matrix <- results_tau$tfr_matrix
-    
-    lambda_c = get.lambda(tfr_matrix_observed, results_tau$T_end_c)
+	suppl.data <- tfr.with.regions$suppl.data
+	if(!is.null(suppl.data$regions)) {
+		suppl.data$tfr_matrix_all <- tfr.with.regions$suppl.data$tfr_matrix
+		suppl.data$tfr_matrix <- results_tau$suppl.matrix
+		suppl.data$T_end <- dim(suppl.data$tfr_matrix)[1]
+		suppl.data$nr_countries <- dim(suppl.data$tfr_matrix)[2]
+	}
+    lambda_c = results_tau$lambda_c
     nr_countries = length(lambda_c)
     nr_countries_estimation <- tfr.with.regions$nr_countries_estimation
-    
-    for (country in 1:nr_countries) {
-        if (lambda_c[country] < results_tau$T_end_c[country]) {
-            updated.tfr.matrix[(lambda_c[country] + 1):results_tau$T_end_c[country],country] = NA
-        }
-    }
-                                               
+                                                   
     # uniform prior for U_c, make lower bound country specific
     tfr_min_c <- apply(updated.tfr.matrix, 2, min, na.rm = TRUE)
     lower_U_c <- ifelse(tfr_min_c > meta$U.c.low.base, tfr_min_c, meta$U.c.low.base)
-    # start_c is the max of tau and 1
-    start_c = results_tau$tau_c
-    start_c[results_tau$id_early] = 1
-	
-	T_end <- dim(tfr.with.regions$tfr_matrix)[1]
-	const_sd_dummie_Tc <- matrix(0, T_end-1, nr_countries)
-	const_sd_dummie_Tc[1:5,] <- 1
-	
+		
 	prop_cov_gammas <- array(NA, c(nr_countries,3,3))
 	if(use.average.gammas.cov) {
 		cov.to.average <- get.cov.gammas(sim.dir=meta$output.dir, burnin=burnin)$values
 		if (all(is.na(cov.to.average))) {
 			warning('Covariance of gamma is NA for all countries. Average from default covariance will be used.', 
 						immediate.=TRUE)
-			data(proposal_cov_gammas_cii)
-			cov.to.average <- proposal_cov_gammas_cii$values
+			e <- new.env()
+			data('proposal_cov_gammas_cii', envir=e)
+			cov.to.average <- e$proposal_cov_gammas_cii$values
 		}
 	} else {
 		# get default proposal_cov_gammas_cii and match with country codes of this run
-    	data(proposal_cov_gammas_cii)
+		e <- new.env()
+    	data('proposal_cov_gammas_cii', envir=e)
     	current.country.codes <- tfr.with.regions$regions$country_code
-    	matched.index <- match(proposal_cov_gammas_cii$country_codes, current.country.codes)
+    	matched.index <- match(e$proposal_cov_gammas_cii$country_codes, current.country.codes)
     	is.notNA <- !is.na(matched.index)
-    	prop_cov_gammas[matched.index[is.notNA],,] <- proposal_cov_gammas_cii$values[is.notNA,,]
+    	prop_cov_gammas[matched.index[is.notNA],,] <- e$proposal_cov_gammas_cii$values[is.notNA,,]
     
 		if (!is.null(proposal_cov_gammas)) { #user-specified, overwrites defaults for given countries
 			matched.index <- match(proposal_cov_gammas$country_codes, current.country.codes)
@@ -218,9 +267,8 @@ do.meta.ini <- function(meta, tfr.with.regions, my.tfr.file=NULL, proposal_cov_g
 			tfr_matrix_all=tfr.with.regions$tfr_matrix_all,
 			tfr_matrix_observed=tfr_matrix_observed,
             tau_c = results_tau$tau_c, lambda_c = lambda_c,
-            const_sd_dummie_Tc=const_sd_dummie_Tc,
             proposal_cov_gammas_cii = prop_cov_gammas,
-            start_c = start_c,
+            start_c = results_tau$start_c, 
             id_Tistau = results_tau$id_Tistau, 
             id_DL = results_tau$id_DL, 
             id_early = results_tau$id_early,
@@ -229,8 +277,10 @@ do.meta.ini <- function(meta, tfr.with.regions, my.tfr.file=NULL, proposal_cov_g
 			U.c.low=lower_U_c,
             nr_countries=nr_countries,
             nr_countries_estimation=nr_countries_estimation,
-            T_end=T_end, T_end_c=results_tau$T_end_c,
-            regions=tfr.with.regions$regions))
+            T_end=dim(tfr.with.regions$tfr_matrix)[1], T_end_c=results_tau$T_end_c, 
+            regions=tfr.with.regions$regions,
+            suppl.data=suppl.data
+            ))
 
 }
 
@@ -254,7 +304,7 @@ mcmc.ini <- function(chain.id, mcmc.meta, iter=100,
     # U_c, the starting levels of the decline
     U_c <- runif(nr_countries, mcmc.meta$U.c.low, mcmc.meta$U.up)
     for (country in mcmc.meta$id_notearly){
-		U_c[country] = mcmc.meta$tfr_matrix[mcmc.meta$tau_c[country],country ]
+		U_c[country] = get.observed.tfr(country, mcmc.meta)[mcmc.meta$tau_c[country]]
 	}
 	
 	##############################################
@@ -303,7 +353,7 @@ mcmc.ini <- function(chain.id, mcmc.meta, iter=100,
         	Triangle_c4[country] = max(mcmc.meta$Triangle_c4.low+0.0001, minf)
           }
     }
-   	dontsave.pars <- c('add_to_sd_Tc', 'meta')
+   	dontsave.pars <- c('add_to_sd_Tc', 'const_sd_dummie_Tc', 'meta')
     if (!save.all.parameters) dontsave.pars <- c(dontsave.pars, 'eps_Tc')
     if (!exists(".Random.seed")) runif(1)	    	
 	mcmc <- structure(list(
@@ -321,6 +371,7 @@ mcmc.ini <- function(chain.id, mcmc.meta, iter=100,
                         output.dir=paste('mc', chain.id, sep=''),
                         traces=0, traces.burnin=0, 
                         rng.state = .Random.seed,
+                        compression.type=mcmc.meta$compression.type,
                         dontsave=dontsave.pars
                         ),
                    class='bayesTFR.mcmc')
@@ -336,16 +387,41 @@ mcmc.ini <- function(chain.id, mcmc.meta, iter=100,
 
 mcmc.meta.ini.extra <- function(mcmc.set, countries=NULL, my.tfr.file = NULL, 
 									burnin = 200, verbose=FALSE) {
+	update.regions <- function(reg, ereg, id.replace, is.new, is.old) {
+		nreg <- list()
+		for (name in c('code', 'area_code', 'country_code')) {
+			reg[[name]][id.replace] <- ereg[[name]][is.old]
+			nreg[[name]] <- c(reg[[name]], ereg[[name]][is.new])
+		}
+		for (name in c('name', 'area_name', 'country_name')) {
+			reg[[name]][id.replace] <- as.character(ereg[[name]])[is.old]
+			nreg[[name]] <- c(as.character(reg[[name]]), 
+									  as.character(ereg[[name]])[is.new])
+		}
+		return(nreg)
+	}
 	meta <- mcmc.set$meta
 	#create tfr matrix only for the extra countries
 	tfr.with.regions <- set.wpp.extra(meta, countries=countries, 
 									  my.tfr.file = my.tfr.file, verbose=verbose)
 	if(is.null(tfr.with.regions)) return(list(meta=meta, index=c()))
+	has.mock.suppl <- FALSE
+	if(is.null(tfr.with.regions$suppl.data$regions) && !is.null(meta$suppl.data$regions)) {
+		# create mock suppl.data in order to get the right data indices
+		nrc <- length(tfr.with.regions$regions$code)
+		mock.suppl <- list(regions=tfr.with.regions$regions, 
+							tfr_matrix=matrix(NA, nrow=nrow(meta$suppl.data$tfr_matrix), ncol=nrc,
+												dimnames=list(rownames(meta$suppl.data$tfr_matrix), NULL)),
+							index.to.all.countries=1:nrc,
+							index.from.all.countries=1:nrc)
+		tfr.with.regions$suppl.data <- mock.suppl
+		has.mock.suppl <- TRUE
+	}
 	Emeta <- do.meta.ini(meta, tfr.with.regions=tfr.with.regions, 
 								my.tfr.file=my.tfr.file, 
 								use.average.gammas.cov=TRUE, burnin=burnin,
 						 		verbose=verbose)
-						 		
+			 		
 	# join the new meta with the existing one
 	is.old <- tfr.with.regions$is_processed
 	is.new <- !tfr.with.regions$is_processed
@@ -366,7 +442,7 @@ mcmc.meta.ini.extra <- function(mcmc.set, countries=NULL, my.tfr.file = NULL,
 					 nr_countries=nr_countries.all
 					)
 					
-	for (name in c('tfr_matrix', 'tfr_matrix_all', 'tfr_matrix_observed', 'const_sd_dummie_Tc')) {
+	for (name in c('tfr_matrix', 'tfr_matrix_all', 'tfr_matrix_observed')) {
 		meta[[name]][,id.replace] <- Emeta[[name]][,is.old]
 		new.meta[[name]] <- cbind(meta[[name]], Emeta[[name]][,is.new])
 	}
@@ -386,20 +462,44 @@ mcmc.meta.ini.extra <- function(mcmc.set, countries=NULL, my.tfr.file = NULL,
 			if (length(not.incl) > 0)
 				meta[[name]] <- meta[[name]][-not.incl]
 		}
+		remove.from.old <- !is.element(idx.old, Emeta[[name]])
+		if(any(remove.from.old)) {
+			idx <- which(is.element(meta[[name]], id.replace[remove.from.old]))
+			meta[[name]] <- meta[[name]][-idx]
+		}
 		new.meta[[name]] <- meta[[name]]
 		idx2 <- !is.inold
 		if(any(idx2))
 			new.meta[[name]] <- c(new.meta[[name]], idx.new.wo.old[Emeta[[name]][idx2]] + meta$nr_countries)
 	}
-	new.meta[['regions']] <- list()
-	for (name in c('code', 'area_code', 'country_code')) {
-		meta$regions[[name]][id.replace] <- Emeta$regions[[name]][is.old]
-		new.meta$regions[[name]] <- c(meta$regions[[name]], Emeta$regions[[name]][is.new])
-	}
-	for (name in c('name', 'area_name', 'country_name')) {
-		meta$regions[[name]][id.replace] <- as.character(Emeta$regions[[name]])[is.old]
-		new.meta$regions[[name]] <- c(as.character(meta$regions[[name]]), 
-									  as.character(Emeta$regions[[name]])[is.new])
+	new.meta[['regions']] <- update.regions(meta$regions, Emeta$regions, id.replace, is.new, is.old)
+	if(!is.null(Emeta$suppl.data$regions) && !has.mock.suppl) {
+		suppl.id.replace <- meta$suppl.data$index.from.all.countries[id.replace]
+		suppl.id.replace <- suppl.id.replace[!is.na(suppl.id.replace)]
+		suppl.is.old <- which(is.old)[which(is.element(meta$suppl.data$index.from.all.countries[id.replace], suppl.id.replace))]
+		suppl.old <- Emeta$suppl.data$index.from.all.countries[suppl.is.old]
+		suppl.is.new <- which(is.new & !is.na(Emeta$suppl.data$index.from.all.countries))
+		suppl.new <- Emeta$suppl.data$index.from.all.countries[suppl.is.new]
+		for (name in c('tfr_matrix', 'tfr_matrix_all')) {
+			meta$suppl.data[[name]][,suppl.id.replace] <- Emeta$suppl.data[[name]][,suppl.old]
+			new.meta$suppl.data[[name]] <- cbind(meta$suppl.data[[name]], Emeta$suppl.data[[name]][,suppl.new])
+		}
+		suppl.is.old.tmp <- rep(FALSE, Emeta$suppl.data$nr_countries)
+		suppl.is.old.tmp[suppl.is.old] <- TRUE
+		new.meta$suppl.data$regions <- update.regions(meta$suppl.data$regions, Emeta$suppl.data$regions, 
+												suppl.id.replace, suppl.new, suppl.old)
+		n.new <- ncol(new.meta$suppl.data$tfr_matrix) - ncol(meta$suppl.data$tfr_matrix)
+		new.meta$suppl.data$nr_countries <- ncol(new.meta$suppl.data$tfr_matrix)
+		new.meta$suppl.data$T_end <- nrow(new.meta$suppl.data$tfr_matrix)
+		new.meta$suppl.data$index.from.all.countries <- meta$suppl.data$index.from.all.countries
+		new.meta$suppl.data$index.to.all.countries <- meta$suppl.data$index.to.all.countries
+		if (n.new > 0) {
+			new.meta$suppl.data$index.from.all.countries <- c(new.meta$suppl.data$index.from.all.countries, rep(NA, sum(is.new)))
+			new.meta$suppl.data$index.from.all.countries[meta$nr_countries + suppl.is.new] <- seq(meta$suppl.data$nr_countries + 1, 
+												length=n.new)
+			new.meta$suppl.data$index.to.all.countries <- c(new.meta$suppl.data$index.to.all.countries, 
+											seq(meta$nr_countries+1, new.meta$nr_countries)[suppl.is.new])
+		} 
 	}
 	index <- id.replace
 	if (new.meta$nr_countries > meta$nr_countries) 
@@ -426,7 +526,7 @@ mcmc.ini.extra <- function(mcmc, countries, index.replace=NULL) {
 	if(nr.countries.extra > nreplace)
 		U_c <- c(U_c, Uc[(nreplace+1):nr.countries.extra])
     for (country in mcmc$meta$id_notearly[is.element(mcmc$meta$id_notearly, countries)]){
-		U_c[country] = mcmc$meta$tfr_matrix[mcmc$meta$tau_c[country],country ]
+		U_c[country] = get.observed.tfr(country, mcmc$meta)[mcmc$meta$tau_c[country]]
 	}
 	mcmc.update <- list(U_c=U_c, d_c=c(mcmc$d_c, rep(mcmc$d.ini, nr.countries.extra-nreplace)), 
 						gamma_ci=rbind(mcmc$gamma_ci, 
