@@ -85,7 +85,8 @@ store.mcmc <- local({
 				} else {
 					values <- buffer[[par]][1:counter,]
 				}
-				write.values.into.file.cindep(par, values, output.dir, mode=open)
+				write.values.into.file.cindep(par, values, output.dir, mode=open, 
+												compression.type=mcmc$compression.type)
 			}
 			country.index <- mcmc$meta$id_DL	
 		} else {
@@ -100,7 +101,8 @@ store.mcmc <- local({
 					values <- buffer.cs[[par]][[country]][1:counter,]
 				}
 				write.values.into.file.cdep(par, values, output.dir, 
-						get.country.object(country, meta=mcmc$meta, index=TRUE)$code, mode=open)
+						get.country.object(country, meta=mcmc$meta, index=TRUE)$code, mode=open, 
+											compression.type=mcmc$compression.type)
 			}
 		}
 		resmc <- as.list(mcmc)
@@ -126,16 +128,28 @@ store.mcmc <- local({
 
 })
 
-do.write.values.into.file <- function(filename, data, mode) {
+.get.compression.settings <- function(compression.type='None') {
+	if(is.null(compression.type)) compression.type <- 'None'
+	return(switch(compression.type,
+							None=c('file', '', ''),
+							xz = c('xzfile', '.xz', 'b'),
+							bz = c('bzfile', '.bz2','b'),
+							gz = c('gzfile', '.gz', 'b')))
+}
+
+do.write.values.into.file <- function(filename, data, mode, compression.type='None') {
+	cmd.suffix.mode <- .get.compression.settings(compression.type)
 	#con <- bzfile(filename, open=mode)
-	con <- file(filename, open=mode)
+	con <- do.call(cmd.suffix.mode[1], list(paste(filename, cmd.suffix.mode[2], sep=''), 
+				open=paste(mode, cmd.suffix.mode[3], sep='')))
 	write.table(data, file=con, row.names=FALSE, col.names = FALSE, sep=" ")
 	close(con)
 }
 
-write.values.into.file.cindep <- function(par, data, output.dir, mode='wb') {
+write.values.into.file.cindep <- function(par, data, output.dir, mode='w', compression.type='None') {
 	#do.write.values.into.file(file.path(output.dir, paste(par,'txt', 'bz2', sep='.')), data, mode=mode)
-	do.write.values.into.file(file.path(output.dir, paste(par,'txt', sep='.')), data, mode=mode)
+	do.write.values.into.file(file.path(output.dir, paste(par,'txt', sep='.')), data, mode=mode, 
+									compression.type=compression.type)
 }
 
 write.table.into.file.cindep <- function(data, ...) {
@@ -143,9 +157,10 @@ write.table.into.file.cindep <- function(data, ...) {
 		write.values.into.file.cindep(par, data[,par], mode='w', ...)
 }
 
-write.values.into.file.cdep <- function(par, data, output.dir, country.code, mode='wb') {
+write.values.into.file.cdep <- function(par, data, output.dir, country.code, mode='w', compression.type='None') {
 	#do.write.values.into.file(file.path(output.dir, paste(par,"_country", country.code, ".txt.bz2",sep = "")), data, mode=mode)
-	do.write.values.into.file(file.path(output.dir, paste(par,"_country", country.code, ".txt",sep = "")), data, mode=mode)
+	do.write.values.into.file(file.path(output.dir, paste(par,"_country", country.code, ".txt",sep = "")), 
+									data, mode=mode, compression.type=compression.type)
 }
 
 write.table.into.file.cdep <- function(data, ...) {
